/*++
 ReadBulk 0.01 by 89ron
--*/

#include "StdAfx.h"

#define CLASS		0xE0		// Wireless Controller
#define SUBCLASS	0x01		// RF Controller
#define PROTOCOL	0x01		// Bluetooth Programming

#define TOUT	100		// Timeout in Milliseconds
#define BUFFER	64		// Bulk Data Size

#define CONSOLE	255

/***************************************
*	Packets Structure
***************************************/
typedef struct Hci {
	unsigned short Header;
	unsigned short Length;
	unsigned char Payload[1];
} HCI, *PHCI; 

typedef struct L2cap {
	unsigned short Length;
	unsigned short channel;				//One channel by sixaxis
	unsigned char Payload[1];
} L2CAP, *PL2CAP;

typedef struct Hidp {
	unsigned char Type;
	unsigned char channel;
	unsigned char Data[1];
} HIDP, *PHIDP;

/***************************************
*	Sixaxis Mapping
***************************************/
#define PS3_BUTTONS				17
#define PS3_ANALOGS				12

#define PS3_JOYS				2
#define PS3_JOYS_AXES			( PS3_JOYS * 2 )

#define PS3_MOTIONS				3 
#define PS3_MOTIONS_STATES		2
#define PS3_MOTIONS_AXES		( PS3_MOTIONS * PS3_MOTIONS_STATES )

typedef struct Sixaxis {
	unsigned char	_none0;
	unsigned short	Digital_Buttons;
	unsigned char	Digital_PsHome;

	unsigned char	_none1;
	unsigned char	Analogue_Joys[PS3_JOYS_AXES];

	unsigned int	_none2;
	unsigned char	Analogue_Buttons[PS3_ANALOGS]; 

	unsigned char	_none3[3];
	unsigned char	infoCable;
	unsigned char	infoBattery;
	unsigned char	infoBT;

	unsigned char	_misc0;
	unsigned char	forceSignal;			//refresh every 5seconds.
	unsigned char	_misc1;
	unsigned short	_misc2;
	
	unsigned char	_misc3;
	unsigned char	_misc4;

	unsigned short	_misc5;
	union {
		char cMotions[PS3_MOTIONS_AXES];
		short sMotions[PS3_MOTIONS];		//sMotions[AccX, AccY, AccZ]
	};

	union {
		unsigned char	cG[2];
		short sGyro;						//if sGyro == 0x0200, Gyro is disabled. 
	};
			

} PS3_MAP, *PPS3_MAP;

/***************************************
*	Source
***************************************/
void fatal(char *msg = usb_strerror())
{
	printf("%s\n",msg); 
	exit(1); 
}
void usage(void)
{
	printf("*** ReadBulk 0.01 via libusb0 by 89ron (c) 2008 ***\n");
	printf("Sniff Sixaxis Bluetooth Packets on USB bulk pipe.\n");
	printf("Require libus0-win32 driver and any version of BtSix.\n");
	printf("\n");
}

int CheckProcess(void)
{
	HANDLE H_Liste_Process; 
	PROCESSENTRY32 The_Process;

	int Int_Process;
	int Num_Process = 0;
	int r = -1;

	H_Liste_Process = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); 

	Int_Process = Process32First(H_Liste_Process, &The_Process); 

	do
	{
		Num_Process++;

		if(strstr(The_Process.szExeFile, "BtSix.exe")) r = 0;
		Int_Process = Process32Next(H_Liste_Process, &The_Process); 
	} 
	while(Int_Process && r);

	return r;
}

void ReadBulk(usb_dev_handle *devh, unsigned char ep)
{
	unsigned char rBuffer[BUFFER];

	int t=0;
	t = usb_bulk_read(devh, ep,(char*) rBuffer, BUFFER, TOUT);

	if( t > 0 )
	{
		for (int i=0;i<t;i++)
			printf("%02x", rBuffer[i]);
		printf("\r");
	}
	else if ( t < 0 )  fatal(); 
}

void Bulk(struct usb_device *dev, int itfnum)
{
	int t=0;

	usb_dev_handle *devh;
	devh = usb_open(dev); 

	printf(" \n++ itfnum: %d\n\n", itfnum);

	if ( usb_set_configuration(devh, 1) < 0 ) fatal();
	else printf("-usb_set_configuration() Pass\n");

	if ( usb_claim_interface(devh, itfnum) < 0 ) fatal(); 
	else printf("-usb_claim_interface() Pass\n");

	printf("\nPress <ESC> for Exit\n\n");
	int Exit=0;
	while(!Exit)
	{
		if(kbhit()) 
			if(getch() == 27) 
				Exit = 1;

		ReadBulk(devh, USB_ENDPOINT_IN | USB_ENDPOINT_TYPE_BULK /* 0x82 BULK IN */ ); 
	};

	if ( usb_release_interface(devh, itfnum) < 0 ) fatal(); 
	else printf("\n\n-usb_release_interface() Pass\n\n");

	usb_close(devh);
}

int main(int argc, char *argv[])
{
	usage();

	if ( CheckProcess() < 0 ) 
		fatal("!!! Launch BtSix and etablished connect, before use Readbulk !!!");

	struct usb_bus *busses; 
	struct usb_bus *bus; 
	int found; 
	
	usb_init(); 
	if ( usb_find_busses() < 0 ) fatal("usb_find_busses"); 
	if ( usb_find_devices() < 0 ) fatal("usb_find_devices"); 
	busses = usb_get_busses(); 
	
	if ( ! busses ) fatal("usb_get_busses"); 
	found = 0; 

	for ( bus=busses; bus; bus=bus->next )
	{
		struct usb_device *dev;
		for ( dev=bus->devices; dev; dev=dev->next)
		{
			int ret;
			char string[256];

			usb_dev_handle *hdev;
			hdev = usb_open(dev) ;
			if ( hdev == NULL ) fatal();

			if (dev->descriptor.iProduct) 
			{
				ret = usb_get_string_simple(hdev, dev->descriptor.iProduct, string, sizeof(string));
				printf("\nProduct : %0s\n", string);
			}
			if(hdev)
				usb_close(hdev);

			printf(" bDeviceClass : 0x%02X\n", dev->descriptor.bDeviceClass);
			printf(" bDeviceSubClass : %d\n", dev->descriptor.bDeviceSubClass);
			printf(" bDeviceProtocol : %d\n", dev->descriptor.bDeviceProtocol);

			if ( dev->descriptor.bDeviceClass == CLASS \
				&& dev->descriptor.bDeviceSubClass == SUBCLASS \
				&& dev->descriptor.bDeviceProtocol == PROTOCOL )
			{
				Bulk(dev, 0);
			}
			++found;
		}
	}
	
	if ( ! found ) printf("No controller found on USB busses."); 
	return 0; 
}
